feature_template = """Feature: Example Feature
  Background: Configure Your Test
    Given some preconditions
    
  Scenario: Test Your App
    Given scenario specific preconditions
    Then verify results
    
"""